/* Copyright 2007 TeamF1,Inc */

/*
modification history
--------------------
01b,12dec07,suh merged changes from FVS336G codebase
01a,02apr07,suh created
*/

/*
 * (C) Copyright 2004,2005
 * Cavium Networks
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/module.h>
#include <linux/kernel.h>

#include "octeon_hal.h"

/**
 * Perform an MII read. Called by the generic MII routines
 *
 * @param dev      Device to perform read for
 * @param phy_id   The MII phy id
 * @param location Register location to read
 * @return Result from the read or zero on failure
 */
static int mii_phy_read
    (
    unsigned char    addr,
    unsigned char    reg,
    unsigned short * pValue
    )
    {
    cvmx_smi_cmd_t     smi_cmd;
    cvmx_smi_rd_dat_t  smi_rd;
    uint64_t           start, end;  /* For timeout */

    smi_cmd.u64 = 0;
    smi_cmd.s.phy_op = 1;
    smi_cmd.s.phy_adr = addr;
    smi_cmd.s.reg_adr = reg;
    cvmx_write_csr (CVMX_SMI_CMD, smi_cmd.u64);

    start = cvmx_get_cycle ();
    end = start + 60000000;     /* 100 mS (assuming 600 MHz) should be enough */

    do
        {
        smi_rd.u64 = cvmx_read_csr (CVMX_SMI_RD_DAT);
        }
    while (smi_rd.s.pending && (cvmx_get_cycle () < end));

    if (smi_rd.s.pending || !smi_rd.s.val)
	return -1;

    *pValue = smi_rd.s.dat;
    return 0;
    }



static int mii_phy_write
    (
    unsigned char  addr,
    unsigned char  reg,
    unsigned short value
    )
    {
    cvmx_smi_cmd_t     smi_cmd;
    cvmx_smi_wr_dat_t  smi_wr;
    uint64_t           start, end;  /* For timeout */

    smi_wr.u64 = 0;
    smi_wr.s.dat = value;
    cvmx_write_csr (CVMX_SMI_WR_DAT, smi_wr.u64);

    smi_cmd.u64 = 0;
    smi_cmd.s.phy_op = 0;
    smi_cmd.s.phy_adr = addr;
    smi_cmd.s.reg_adr = reg;
    cvmx_write_csr (CVMX_SMI_CMD, smi_cmd.u64);

    start = cvmx_get_cycle ();
    end = start + 60000000;     /* 100 mS (assuming 600 MHz) should be enough */

    do
        {
        smi_wr.u64 = cvmx_read_csr (CVMX_SMI_WR_DAT);
        }
    while (smi_wr.s.pending && (cvmx_get_cycle () < end));

    if (smi_wr.s.pending)
	return -1;

    return 0;
    }


static int __init mii_init (void)
    {
    printk (KERN_INFO "Loading %s\n", __FILE__);
    return 0;
    }

static void __exit mii_exit (void)
    {
    printk (KERN_INFO "Unloading %s\n", __FILE__);
    }


EXPORT_SYMBOL(mii_phy_read);
EXPORT_SYMBOL(mii_phy_write);
module_init(mii_init);
module_exit(mii_exit);
